-- 
-- Written by: Vincent LARK
--                           vincent.lark@gmail.com
--   
-- This program is free software; you can redistribute it and/or
-- modify it under the terms of the GNU Library General Public License
-- as published by the Free Software Foundation; either version 2 of
-- the License, or (at your option) any later version.
-- 
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU Library General Public License for more details.
-- 
-- You should have received a copy of the GNU Library General Public
-- License along with this program; if not, write to the Free Software
-- Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
--

class AUDIO_COLLECTION
-- This class represents an audio collection
-- provides basic operations like add_directory

	
creation
	make
	

feature {NONE}

	audio_files : ARRAY[AUDIOFILE]
	-- Representation de la collection
	
	audioutils : AUDIOUTILS

	factory : AUDIOFILE_FACTORY
	
feature

	make is
	-- Constructeur de la collection
		do
			create audio_files.make(0,0)
			audio_files.clear_count
			create audioutils.make
			create factory.make
		end

	
	lower : INTEGER is
	-- Retourne le premier index
	-- de la collection
		do
			Result := audio_files.lower
		end
	

	count : INTEGER is
	-- Nombre de fichiers prsents
	-- dans la collection
		do
			Result := audio_files.count
		end


	
	valid_index(idx : INTEGER) : BOOLEAN is
	-- L'index spcifi est il valide ?
		do
			Result := audio_files.valid_index(idx)
		end

	
	
	item(position : INTEGER) : AUDIOFILE is
	-- Retourne l'objet AUDIOFILE a
	-- la position 'position'
		require
			audio_files.valid_index(position)
		do
			Result := audio_files.item(position)
		end
	

	
	add_playlist(path : STRING) : INTEGER is
	-- Ajoute les fichiers dcrits dans
	-- le fichier path (uniquement les fichiers reconnus)
		require
			path /= Void
			not path.is_empty
		local
			tmp_files_array : ARRAY[STRING]
		do
			--Result := 0

			tmp_files_array := audioutils.get_playlist_entries(path)

			if (not tmp_files_array.is_empty) then
				Result := add_audiofile_array(tmp_files_array)
			end
		ensure
			Result >= 0
		end


	
	add_directory(path : STRING ; recursive : BOOLEAN) : INTEGER is
	-- Ajoute le rpertoire (uniquement les fichiers reconnus)
		require
			path /= Void
			not path.is_empty
		local
			tmp_files_array : ARRAY[STRING]
		do
			--Result := 0

			tmp_files_array := audioutils.scan_for_audio_files(path, recursive)

			if (not tmp_files_array.is_empty) then
				Result := add_audiofile_array(tmp_files_array)
			end
		ensure
			Result >= 0
		end


	
	add_audiofile_array(file_array : ARRAY[STRING]) : INTEGER is
	-- Ajoute le tableau de fichiers 'file_array'
		require
			file_array /= Void
		local
			tmp_audiofile : AUDIOFILE
			ptr_array : INTEGER
			tmp_file_path : STRING
			file_tools : FILE_TOOLS
		do
			--Result := 0

			from
				ptr_array := file_array.lower
			until ptr_array = file_array.upper
			loop

				tmp_file_path := file_array.item(ptr_array)
				
				if not file_tools.file_exists(tmp_file_path) or
					not file_tools.is_readable(tmp_file_path)
				 then
					io.put_string("Le fichier ")
					io.put_string(tmp_file_path)
					io.put_string(" n'existe pas ou n'est pas lisible !")
					io.put_new_line
				
				else
					tmp_audiofile := factory.create_appropriate(tmp_file_path)
					audio_files.add_last(tmp_audiofile)

					Result := Result + 1
				end
				
				ptr_array := ptr_array + 1
			end
		ensure
			Result >= 0
		end -- End add_audiofile_array 



	fast_random_audiofile_index : INTEGER is
	-- Choisi un fichier au hasard
		require
			Current.count > 0
		do
			Result := audioutils.random_number(Current.count) - 1
		ensure
			Result.in_range(0, Current.count - 1)
			Current.valid_index(Result)
		end
	


	tagged_random_audiofile_index(n : INTEGER) : INTEGER is
	-- Choisi un fichier taggu au hasard
	-- tente n fois, puis s'arrete s'il n'est pas trouv
	-- REMARQUE: si n = -1, aucune limite (attention!)
	--           retourne -1 si non trouv
		local
			try_num, tmp_index : INTEGER
			found : BOOLEAN
		do
			found := False
			Result := -1

			from try_num := 1
			until found
				or try_num = n
			loop
				tmp_index := audioutils.random_number(Current.count - 1)

				if audio_files.item(tmp_index).is_tag_present_and_not_empty then
					found := True
					Result := tmp_index
				else
					try_num := try_num + 1
				end
			end
		ensure
			Result < Current.count
			Result >= 0 or Result = -1
		end
	
end
