-- 
-- Written by: Vincent LARK  vincent.lark@gmail.com
--             Julien COLIN
--   
-- This program is free software; you can redistribute it and/or
-- modify it under the terms of the GNU Library General Public License
-- as published by the Free Software Foundation; either version 2 of
-- the License, or (at your option) any later version.
-- 
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU Library General Public License for more details.
-- 
-- You should have received a copy of the GNU Library General Public
-- License along with this program; if not, write to the Free Software
-- Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
--

class AUDIOFILE_FACTORY
-- This class represent a audio types dictionary, and dynamic constructor

	
--inherit SIMPLE_DICTIONARY[ STRING , FUNCTION[ANY, TUPLE[STRING], AUDIOFILE]]
--		rename make as make_super end

creation
	make
		
feature {NONE}

	mykey : FUNCTION[ANY, TUPLE[STRING], AUDIOFILE]
	myvalue : STRING


	keys_array : ARRAY[like mykey]
	values_array : ARRAY[like myvalue]

feature

	
	make is
		-- Initialisation de l'objet
		do
			create keys_array.make(0,2)
			create values_array.make(0,2)

			keys_array.clear_count
			values_array.clear_count

			-- On ajoute le type de fichier avec
			-- le constructeur correspondant
			-- REMARQUE: le chemin (ncessaire au constructeur)
			--           n'est pas encore connu (prsence du 'agent ... ?').
			add_audio_constructor("MP3", agent create_mp3_audiofile(?))

			-- Le support des fichiers OGG, c'est pour bientt !
			--add_audio_constructor("OGG", agent create_ogg_audiofile(?))
			
		end

	

	add_audio_constructor(audiotype : like myvalue ; constructor : like mykey) is
	-- Fonction d'ajout des constructeurs dynamiques
		require
			audiotype /= Void
			constructor /= Void
		do
			-- On place ici le type de fichier
			-- avec le constructeur correspondant.
			-- dans les deux tableaux
			-- ATTETION: il FAUT que audiotype et constructor se
			--            trouve au meme index sur chaque tableau !!
			values_array.add_last(audiotype)
			keys_array.add_last(constructor)
		end

	

	query(extension : STRING) : STRING is
	-- Retourne le type correspondant
	--  l'extension 'extension'
		require
			extension /= Void
		do
			extension.to_upper
			
			if (Current.has(extension)) then
				Result := extension
			else
				Result := ""
			end
		end


	
	-- Cre l'objet MP3 (utilisation dynamique)
	create_mp3_audiofile(path: STRING) : MP3 is
		require
			path /= Void
			not path.is_empty
		local
			tmp_mp3: MP3
		do
			create tmp_mp3.make(path)
			Result := tmp_mp3.twin
		end

	

	create_ogg_audiofile(path: STRING) : OGG is
	-- Cre l'objet OGG (utilisation dynamique)
		require
			path /= Void
			not path.is_empty
		local
			tmp_ogg: OGG
		do
			create tmp_ogg.make(path)
			Result := tmp_ogg.twin
		end
	

	
	create_appropriate(path : STRING) : AUDIOFILE is
	-- Cration "pseudo dynamique" de l'objet AUDIOFILE
		require
			path /= Void
			not path.is_empty
		local
			delayed_constructor : like mykey
			audio_type : STRING
			audioutils : AUDIOUTILS
		do
			create audioutils.make

			-- D'aprs l'extension du fichier audio ....
			audio_type := Current.query(audioutils.file_extension(path))

			-- ... on trouve le constructeur associ ... 
			if Current.has(audio_type) then
				delayed_constructor := Current.at(audio_type)

				-- ... et on appelle cette fonction avec le chemin du fichier
				-- (d'aprs le tutorial  SmartEifel/tutorial/agent/example6.e)
				Result := delayed_constructor.item([path])
			end
		end


	
	has(v : like myvalue) : BOOLEAN is
	-- Simple fonction de test de prsence de cl
		require
			v /= Void
			values_array /= Void
		do
			Result := values_array.has(v)
		end

	
	
	at(v : like myvalue) : like mykey is
	-- Trouver la fonction correspondant  la cl
		require
			v /= Void
			values_array /= Void
			keys_array /= Void
			keys_array.count = values_array.count
		do
			Result := keys_array.item(values_array.index_of(v))
		end



end

