expanded class CONFIG_FILE



feature {NONE}

	config_file_path : STRING is "config.txt"
	-- Chemin vers le fichier de configuration
	

	read_entry_real(entry : STRING) : STRING is
	-- Lire une entree dans le fichier de conf
		require
			entry /= Void
			not entry.is_empty
		local
			file : TEXT_FILE_READ
			line_buffer : STRING
			found, error : BOOLEAN
		do
			create file.connect_to(config_file_path)
			create line_buffer.make_empty
			
			if file.is_connected then
				
				from
				until found or error or file.end_of_input
				loop
					if file.can_read_line then
						file.read_line
						
						line_buffer := file.last_string
						
						if line_buffer /= Void then
							if not line_buffer.is_empty  then
								
								line_buffer.right_adjust
								line_buffer.left_adjust

								if not line_buffer.first.same_as('#')
									and line_buffer.has_prefix(entry)
								 then
									found := True

									Result := line_buffer.substring(entry.count + 2, line_buffer.upper).twin
								end
							end
						end

						line_buffer.wipe_out
					else
						error := True
					end
				end

				file.disconnect
				
			else
				io.put_string("Le fichier de configuration (")
				io.put_string(config_file_path)
				io.put_string(") est introuvable !")
			end

		end
	


	high_scores_real : ARRAY[HIGHSCORE] is
			-- Rcupre les scores crits dans
			-- le fichier de configuration
		local
			file : TEXT_FILE_READ
			line_buffer : STRING
			error : BOOLEAN
			tmp_hs : ARRAY[STRING]
		do
			create file.connect_to(config_file_path)
			create line_buffer.make_empty

			create Result.make(0, 0)
			Result.clear_count

			if file.is_connected then
				
				from
				until file.end_of_input or error
				loop
					if file.can_read_line then
						file.read_line
						
						line_buffer := file.last_string
						
						if line_buffer /= Void then
							if not line_buffer.is_empty  then
								
								line_buffer.right_adjust
								line_buffer.left_adjust

								if not line_buffer.first.same_as('#')
									and line_buffer.has_prefix("%%")
								 then

									tmp_hs := line_buffer.twin.split

									-- On teste si l'entree de highscore est 
									-- valide (cad si elle contient 4 elements)
									-- (sinon le fichier est corrompu)
									if tmp_hs.count >= 4 then

										-- On teste si les 2 dernieres entrees
										-- sont effectivement des integer
										if tmp_hs.item(tmp_hs.lower + 2).is_integer and
											tmp_hs.item(tmp_hs.lower + 3).is_integer
										 then
										
											-- On teste si le nb de bonnes rponses 
											-- est gal ou infrieur  celui de 
											-- toutes les rponses
											-- (sinon le fichier est corrompu)
											if tmp_hs.item(tmp_hs.lower + 2).to_integer <=
												tmp_hs.item(tmp_hs.lower + 3).to_integer
											 then

												Result.add_last(create {HIGHSCORE}.
																	 make(tmp_hs.item(tmp_hs.lower + 1),
																			tmp_hs.item(tmp_hs.lower + 2).to_integer,
																			tmp_hs.item(tmp_hs.lower + 3).to_integer)
																	 )
											else
											   error := True
											end
											
										else
										   error := True
										end
										
									else
										error := True
									end

									if error then
										io.put_string("Erreur dans le fichier de config !")
										io.put_new_line
									end
									
								end
							end
						end

						line_buffer.wipe_out
					else
						error := True
					end
				end

				file.disconnect
				
			else
				io.put_string("Le fichier de configuration (")
				io.put_string(config_file_path)
				io.put_string(") est introuvable !")
			end

		end -- end of high_scores_real
	

	
	write(entry : STRING) : BOOLEAN is
		-- Ecrire une ligne dans le fichier de configuration
		require
			entry /= Void
		local
			file : TEXT_FILE_WRITE
		do
			create file.connect_for_appending_to(config_file_path)

			if file.is_connected then
				file.put_string(entry)
				file.put_new_line
				Result := True
				file.disconnect
			end

		end

	
	
feature

	
	read_entry(entry : STRING) : STRING is
			-- Lire la valeur de l'entree
		require
			entry /= Void
			not entry.is_empty
		do
			Result := read_entry_real(entry)
		end


	
	high_scores : ARRAY[HIGHSCORE] is
			-- Tableau des meileurs scores
		do
			Result := high_scores_real
		end
	
	

	write_highscore(name : STRING ;
						  nb_good_responses : INTEGER ;
						  nb_responses : INTEGER) : BOOLEAN is
			-- Ecrire une entree de type score
		require
			name /= Void
			not name.is_empty
		local
			tmp_entry : STRING
		do
			tmp_entry := "%% " + name
				+         " " + nb_good_responses.to_string
				+         " " + nb_responses.to_string
			
			Result := write(tmp_entry)
		end
	
	
end
