-- 
-- Written by: Vincent LARK vincent.lark@gmail.com
--             Julien COLIN
--   
--   
-- This program is free software; you can redistribute it and/or
-- modify it under the terms of the GNU Library General Public License
-- as published by the Free Software Foundation; either version 2 of
-- the License, or (at your option) any later version.
-- 
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU Library General Public License for more details.
-- 
-- You should have received a copy of the GNU Library General Public
-- License along with this program; if not, write to the Free Software
-- Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
--

class GAME

creation
	make_with_audio_collection, make_with_path, make_with_playlist

	
feature {NONE}

	processus_audio: PROCESS
	-- Processus de lecture audio

	audio_collection : AUDIO_COLLECTION
	-- Blibliotheque audio associe au jeu

	audioutils : AUDIOUTILS
	-- Objet AUDIO_UTILS

	system : SYSTEM
	-- Objet SYSTEM, pour lancer et arreter la lecture du fichier

	config : CONFIG_FILE
	-- Objet CONFIG


	nb_try_changed : BOOLEAN


	affecte_a_l_entree(index : INTEGER ; pos : INTEGER) is
		-- Affecte une rponse  une position donne
		require
			index.in_range(0, audio_collection.count)
			audio_collection.item(index).is_tag_present
			pos.in_range(random_titles_array.lower, random_titles_array.upper)
		local
			entree_complete : STRING
		do
			create entree_complete.make(0)

			entree_complete.append("[ ")
			entree_complete.append(audio_collection.item(index).artist)
			entree_complete.append(" - ")
			entree_complete.append(audio_collection.item(index).title)
			entree_complete.append(" ]")

			random_titles_array.put(entree_complete, pos)
		end


	
	no_tagged_audiofiles_enough is
			-- Si le nombre de fichiers audio
			-- est insuffisant, on arrete. (C'est trop bourin !)
		do
			io.put_new_line
			io.put_string(" Pas assez de fichiers taggues ! la recherche n'a pas abouti.")
			io.put_new_line
			is_game_finished := True
		end
	
	
	
feature

	count : INTEGER
	-- Nombre de fichiers

	score : INTEGER
	-- Score du joueur

	nb_choices : INTEGER
	-- Nombre de choix possibles

	nb_try : INTEGER
	-- Nombre d'essais pour trouver
	-- un fichier corretement taggu

	is_game_finished : BOOLEAN
	-- Le jeu est-il fini ?
	
	last_good_position, last_good_index : INTEGER
	-- Dernieres bonnes rponses
	
	random_titles_array : ARRAY[STRING]
	-- Tableau des rponses possibles


	sample_playing : BOOLEAN
	-- Renvoie VRAI lorsque le fichier est
	-- en train dtre jou.


	-- Nombre de rponses donnes
	nb_responses : INTEGER

	
	nb_good_responses : INTEGER
	-- Nombre de bonnes rponses donnes

	
	

	
	make_common is
			-- Initialisation commune aux constructeurs
		require
			not sample_playing
		do
			is_game_finished := False

			-- Rinitilalisation des scores
			nb_responses := 0
			nb_good_responses := nb_responses
			nb_choices := 2
			
			-- ... on est moyen gentils ?
			-- on offre (ou pas) un score initial ?
			score := 2
			
			-- Valeur par dfaut de nb_try
			if not nb_try_changed then
				nb_try := -1
			end
			nb_try_changed := False
			
		ensure
			not is_game_finished
		end

	
	
	make_with_audio_collection(audio_collec : AUDIO_COLLECTION) is
			-- Constructeur ( partie d'une collection dja faite)
		require
			audio_collec /= Void
		do
			make_common
			audio_collection := audio_collec
			count := audio_collection.count

			if count < 2 then
				no_tagged_audiofiles_enough
			end

		ensure
			audio_collection /= Void
			not is_game_finished
		end

	
	
	make_with_playlist(playlist : STRING) is
	-- Constructeur ( partie d'une collection dja faite)
		require
			playlist /= Void
			not playlist.is_empty
		do
			make_common
			create audio_collection.make
			count := audio_collection.add_playlist(playlist)

			if count < 2 then
				no_tagged_audiofiles_enough
			end

		ensure
			audio_collection /= Void
			not is_game_finished
		end

	
	
	make_with_path(path : STRING ; recurs : BOOLEAN) is
	-- Constructeur ( partir d'un rpertoire)
		require
			path /= Void
			not path.is_empty
		do
			make_common
			create audio_collection.make
			count := audio_collection.add_directory(path, recurs)

			if count < 2 then
				no_tagged_audiofiles_enough
			end

		end


	play_sample is
	-- Lance la lecture du fichier audio
		require
			audio_collection /= Void
			audio_collection.valid_index(last_good_index)
			not sample_playing
		local
			path, file : STRING
			pos : INTEGER
			--i : INTEGER
			--command : STRING
			--args : ARRAY[STRING]
		do
			-- On lit le fichier de config
			-- pour trouver la commande a lancer
			path := config.read_entry("START_COMMAND")

			if path /= Void then
				
				if not path.is_empty then
					
					file := audio_collection.item(last_good_index).path.twin
					file.add_first('"')
					file.add_last('"')

					-- Avec un objet PROCESS .... je n'y arrive pas !					
					--args := path.split
					--command := args.item(args.lower)
					--args.remove_first
					--args.replace_all("%"@%"", file)
					
					--io.put_string(command + " ")
					--from i := args.lower
					--until i > args.upper
					--loop
					--	io.put_string(args.item(i) + " ")
					--	i := i + 1
					--end
					--io.put_new_line					
					--create processus_audio.execute(command, args , True)

					pos := path.first_substring_index("%"@%"")
					path.replace_substring (file, pos, pos + 2)					

					sample_playing := True

					system.execute_command_line(path)
				end
			end
			
		end -- end of play_sample
	
	
	
	stop_sample is
	-- Arreter la lecture du fichier audio
		require
			sample_playing
		local
			path : STRING
		do
			-- On lit le fichier de config
			-- pour trouver la commande a lancer
			path := config.read_entry("STOP_COMMAND")
			system.execute_command_line(path)
			sample_playing := False

			-- Avec un objet PROCESS ... je n'y arrive pas !
			-- if processus_audio.is_connected then
			--	-- processus_audio.kill(9)
			--	-- processus_audio.wait
			--	   sample_playing := False
			-- end

		ensure
			sample_playing = False
		end




	
feature

	
	init_game_choices(nbc : INTEGER) is
			-- Initialise un tableau de jeu
			-- et renvoie la bonne rponse
		require
			nb_choices > 1
		local
			entree_correct : BOOLEAN
			tmp_integer_choices : ARRAY[INTEGER]
			random_index, random_position, entree_choix, other_audiofile_index : INTEGER
		do
			nb_choices := nbc

			create tmp_integer_choices.make(0, nb_choices)
			create random_titles_array.make(0, nb_choices)

			
			-- On choisi un fichier audio au hasard
			random_index := audio_collection.tagged_random_audiofile_index(nb_try)
			
			-- Si non trouv, on estime qu'il n'y a pas
			-- asssez de fichiers
			if random_index = -1 then
				no_tagged_audiofiles_enough
			else
	
				-- On choisi une position (dans les rponses proposes)
				random_position := audioutils.random_number(nb_choices)
				
				-- Pour chaque entre rponse, on choisi un titre
				from entree_choix := random_titles_array.lower
				until entree_choix = random_titles_array.lower + nb_choices
				loop

					-- Si entree_choix correspond  la position
					-- alatoirement choisie ...
					if entree_choix = random_position then
						
						-- ... alors on place ici la bonne rponse
						affecte_a_l_entree(random_index, entree_choix)

						-- On place galement cet index dans le tableau
						-- temporaire, pour tester son unicit
						tmp_integer_choices.put(random_index, entree_choix)
						
						entree_choix := entree_choix + 1

					else
						
						-- Sinon, cration d'une rponse possible ...
						from entree_correct := False
						until entree_correct
						loop
							
							-- On prend un autre fichier au hasard ... 
							other_audiofile_index := audio_collection.tagged_random_audiofile_index(nb_try)

							-- S'il aucun fichier taggu n'a t trouv...
							if other_audiofile_index = -1 then
								no_tagged_audiofiles_enough
							else
								
								-- Tests d'unicit et de validit (title et artist /= "" )
								-- On place son index dans le tableau temporaire afin
								-- de tester son unicit
								tmp_integer_choices.put(other_audiofile_index.twin,
																entree_choix)
								
								entree_correct := (tmp_integer_choices.occurrences(other_audiofile_index) = 1)				

								if entree_correct then
									
									--io.put_string(audio_collection.item(other_audiofile_index).path)
									--io.put_new_line
									
									-- ... et on affecte son index a une
									-- des rponses possibles
									affecte_a_l_entree(other_audiofile_index.twin,
															 entree_choix)

									entree_choix := entree_choix + 1
								end

								--from i := tmp_integer_choices.lower 
								--until i = tmp_integer_choices.upper
								--loop
								--	io.put_integer(tmp_integer_choices.item(i))
								--	io.put_string(" | ")
								--	i := i + 1
								--end
								--io.put_new_line


								
							end
							
						end

					end -- end of if random_position = random_index
				
				end -- end of random_titles_array making

			end -- end of if random_index = -1

			-- On place la bonne rponse dans la variable 
			-- last_good_position et last_good_index
			last_good_position := random_position
			last_good_index := random_index
			
		end -- end of init_game_choices



	change_nb_try(new_bn_try : INTEGER) is
			-- Change le nombre d'essai
			-- pour la recherche.
		require
			new_bn_try > 0 or new_bn_try = -1
		do
			nb_try := new_bn_try.twin
		end
			

	
	
	is_the_answer(ans : INTEGER) : BOOLEAN is
			-- Teste si cette rponse est la bonne
			-- et met  jour le score
		require
			not is_game_finished
			audio_collection /= Void
			ans.in_range(0, nb_choices)
		do
			Result := (ans = last_good_position)

			nb_responses := nb_responses + 1

			if Result then
				score := score + 1
				nb_good_responses := nb_good_responses + 1
			else
				score := score - 1

				if score = 0 then
					is_game_finished := True
				end
				
			end
		end -- end of is_the_answer



	
	write_highscore(name : STRING) is
	-- Ecrire le highscore
		require
			name /= Void
			not name.is_empty
			is_an_highscore
		local
			write_ok : BOOLEAN
		do
			write_ok := config.write_highscore(name, nb_good_responses, nb_responses)
		end -- end of write_highscore



	high_scores : ARRAY[HIGHSCORE] is
	-- Retourne le tableau des meilleurs scores
		do
			Result := config.high_scores
		end
	


	sorted_high_scores : ARRAY[HIGHSCORE] is
	-- Retourne le tableau TRIE des meilleurs scores
		local
			sorter: REVERSE_COLLECTION_SORTER[HIGHSCORE]
		do
			Result := config.high_scores
			
			sorter.sort(Result)
			check
				sorter.is_sorted(Result)
			end
		end
	

	is_an_highscore : BOOLEAN is
	-- Teste si la dernire rponse est la bonne
		local
			place, i, tmp_score : INTEGER
			hs : ARRAY[HIGHSCORE]
		do
			hs := config.high_scores
			place := 1
			Result := False

			if hs.is_empty then
				Result := True
			else

				from i := hs.lower
				until i > hs.upper
				loop
					-- Le nombre de bonnes reponses est stock l
					tmp_score := hs.item(i).nb_good_responses

					if Current.nb_good_responses <= tmp_score then
						place := place + 1
					end

					i := i + 1
				end

				if place <= 3 then
					Result := True
				end
			end
		end

	
	
invariant
	nb_try > 0 or nb_try = -1
end

