-- 
-- Written by: Vincent LARK  vincent.lark@gmail.com
--             Julien  COLIN 
--   
--   
-- This program is free software; you can redistribute it and/or
-- modify it under the terms of the GNU Library General Public License
-- as published by the Free Software Foundation; either version 2 of
-- the License, or (at your option) any later version.
-- 
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU Library General Public License for more details.
-- 
-- You should have received a copy of the GNU Library General Public
-- License along with this program; if not, write to the Free Software
-- Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
--

class MP3

inherit AUDIOFILE redefine title, artist, genre, album, annee, comment end
	
creation
	make
	
feature {}

	-- Emplacement des differente parties du tag
	ptitre   : INTEGER is 3
	partiste : INTEGER is 33
	palbum   : INTEGER is 63
	pannee   : INTEGER is 93
	pcomm    : INTEGER is 97
	pgenre   : INTEGER is 127
	tannee   : INTEGER is 4

	
	t_ti_ar_al_com : INTEGER is 30
			-- Taille commune pour le titre, l'artiste,l'album et le commentaire

	tag_lenght : INTEGER is 128
	
	tag_array : ARRAY[CHARACTER] -- Array of the 128 last bytes.


	
	tag_zone(start_index : INTEGER; zone_lenght: INTEGER) : STRING is
			-- Retourne la partie du tag brut commencant 
			-- a l'indice start_index et de taille zone_lenght.
		require
			start_index.in_range(0, tag_lenght)
			zone_lenght > 0
		local
			ptr_zone : INTEGER
			temp_tag : ARRAY[CHARACTER]
		do
			temp_tag := tag_array.slice(start_index, start_index + zone_lenght)
			create Result.make_empty
			
			from
				ptr_zone := temp_tag.lower
			until
				ptr_zone = temp_tag.upper
			loop
				if (temp_tag.item(ptr_zone).is_ascii) and
					not (temp_tag.item(ptr_zone).same_as('%U'))
				 then
					Result.append(temp_tag.item(ptr_zone).to_string)
				end
				
				ptr_zone := ptr_zone + 1
			end

			Result.left_adjust
			Result.right_adjust
			
		end -- End of tag_zone
	

	
feature
	
	make(location: STRING) is
			-- Constructeur MP3, le chemin du fichier 
			-- en paramtre
		do
			path := location
			create file.make
			create tag_array.make(0, tag_lenght)

			--taille := file_lenght
		end -- End of make

	

	is_tag_present_real : BOOLEAN is
			-- Retourne Vrai si le fichier MP3
			-- contient un tag
		local
			i : INTEGER
		do
			file.connect_to(path)

			-- Parcours le fichier
			-- jusqu'a la zone tag (file_lenght - tag_lenght)
			-- (-1 parce qu'on lit un premier caractre par la suite.
			from i := 0
			until file.end_of_input or i > file_lenght - tag_lenght - 1
			loop
				file.read_byte
				i := i + 1
			end

			-- On rempli le tableau tag_array
			-- avec les 128 derniers octets. 
			from i := 0
			until file.end_of_input or i = tag_lenght
			loop
				file.read_byte
				tag_array.put(file.last_byte.to_character, i)
				i := i + 1
			end
			
			-- La partie tag du fichier
			-- dbute par les caractres T,A,G.
			Result := tag_array.item(0).same_as('T')
				and tag_array.item(1).same_as('A')
				and tag_array.item(2).same_as('G')
			
			file.disconnect
			
		end -- End of is_tag_present_real


	

	raw_tag_string : STRING is
			-- Retourne la partie TAG du fichier, en entier
		require
			is_tag_present
		do
			Result := tag_zone(tag_array.lower, tag_array.upper)
		end


	title : STRING is
			-- Tag 'titre'
		require
			is_tag_present
		do
			Result := tag_zone(ptitre, t_ti_ar_al_com)
		end

	
	artist : STRING is
			-- Tag 'artist'
		require
			is_tag_present
		do
			Result := tag_zone(partiste, t_ti_ar_al_com)
		end

	
	album : STRING is
			-- Tag 'album'
		require
			is_tag_present
		do
			Result := tag_zone(palbum, t_ti_ar_al_com)
		end

	
	annee : INTEGER is
			-- Tag 'anne'
		require
			is_tag_present
		local
			string_year : STRING
		do
			string_year := tag_zone(pannee, tannee)
			if (string_year.is_integer) then
				Result := string_year.to_integer
			else
				Result := 0
			end
		end
	

	genre : INTEGER is
			-- Tag 'genre'
		require
			is_tag_present
		do
			Result := tag_array.item(pgenre).to_integer
		end
	

	comment : STRING is
			-- Tag 'commentaire'
		require
			is_tag_present
		do
			Result := tag_zone(pcomm, t_ti_ar_al_com)
		end
		
	
end
	
