-- 
-- Written by: Vincent LARK  vincent.lark@gmail.com
--             Julien COLIN
--   
--   
-- This program is free software; you can redistribute it and/or
-- modify it under the terms of the GNU Library General Public License
-- as published by the Free Software Foundation; either version 2 of
-- the License, or (at your option) any later version.
-- 
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU Library General Public License for more details.
-- 
-- You should have received a copy of the GNU Library General Public
-- License along with this program; if not, write to the Free Software
-- Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
--

class OGG

inherit AUDIOFILE redefine title, artist, genre, album, annee, comment end
	
creation
	make
	
feature {}

	tag_array : ARRAY[ARRAY[CHARACTER]]
	-- Unlimited user comment fields.

	tag_lenght : INTEGER

	tag_lenghts : ARRAY[INTEGER]

	vendor : ARRAY[CHARACTER]

	
	actual_tag_lenght : INTEGER is
		do
			file.connect_to(path)
			Result := 0

			if not file.end_of_input then
				file.read_integer_32_native_endian
				Result := file.last_integer_32
			end
			file.disconnect
		end
	

	
	tag_query(tag : ARRAY[CHARACTER]; count : INTEGER) : INTEGER is
		local
			i, fulltaglen, found : INTEGER
			fulltag : ARRAY[CHARACTER]
		do
			i := 0
			found := 0
			fulltaglen := tag.count + 1
			create fulltag.make(0, fulltaglen)
			Result := -1

			fulltag.copy(tag)
			fulltag.add_last('=')
			
			from
				i := 0
			until
				i = tag_lenght
			loop
				if (tags_are_equals(tag_array.item(i), fulltag, fulltaglen)) then
					if (count = found) then
						Result := i
					else
						found := found + 1
					end
				end
				i := i + 1
			end

		end


	
	string_tag_query(tag : STRING; count : INTEGER) : STRING is
		do
			Result := "abcdefg"
		end

	
	
	string__tag__query(tag : STRING; count : INTEGER) : STRING is
		local
			ptr_tag, tag_id, tmp_lenght : INTEGER
			tmp_tag_array : ARRAY[CHARACTER] -- ! 2 use
			found : BOOLEAN
		do
			create Result.make_empty
			create tmp_tag_array.make(0, tag.count)

			from
				ptr_tag := tag.lower
			until
				ptr_tag = tag.count
			loop
				tmp_tag_array.put(tag.item(ptr_tag), ptr_tag)
				ptr_tag := ptr_tag + 1
			end
			
			tag_id := tag_query(tmp_tag_array, count)
			tmp_tag_array.clear_count
			
			if (tag_id >= 0) then
				
				tmp_lenght := tag_lenghts.item(tag_id)
				
				create tmp_tag_array.make(0, tmp_lenght)
				tmp_tag_array.copy(tag_array.item(tag_id))

				from
					ptr_tag := 0
					found := False
				until
					ptr_tag = tmp_lenght or found
				loop
					if (tmp_tag_array.item(ptr_tag).same_as('=')) then

						from
						until 
							ptr_tag = tmp_lenght
						loop
							Result.append_character(tmp_tag_array.item(ptr_tag))
						end

					end
				end
				
			end
		end
		

	
  tags_are_equals(tag1 : ARRAY[CHARACTER];
  -- This is more or less the same as strncasecmp
						tag2 : ARRAY[CHARACTER];
						long : INTEGER) : BOOLEAN is
		require
			tag1 /= Void
			tag1.count >= long

			tag2 /= Void
			tag2.count >= long
		local
			ptr : INTEGER
			c1, c2 : INTEGER
		do
			Result := True
			from
			until
				ptr = long
				or not Result
			loop
				c1 := tag1.item(ptr).to_integer
				c2 := tag2.item(ptr).to_integer

				if ('Z'.to_integer >= c1) and (c1 >= 'A'.to_integer) then
					c1 := c1 - 'A'.to_integer + 'a'.to_integer
				end
				
				if ('Z'.to_integer >= c2) and (c2 >= 'A'.to_integer) then
					c2 := c2 - 'A'.to_integer + 'a'.to_integer
				end

				if (c1 /= c2) then
					Result := False
				end
				
				ptr := ptr + 1
			end
		end


	
feature
	
	-- Constructeur OGG
	--
	make(location: STRING) is
		do
			path := location
			create file.make
			create tag_array.make(0, tag_lenght)

			--taille := file_lenght
			tag_lenght := actual_tag_lenght
		end

	
	-- Retourne Vrai si le fichier
	-- contient un tag
	is_tag_present_real : BOOLEAN is
		do
			Result := (tag_lenght > 0)
		end


	-- Recuperation des tags
	--
	title : STRING is
		do
			Result := string_tag_query("TITLE", 0)
		end

	
	artist : STRING is
		do
			Result := string_tag_query("ARTIST", 0)
		end

	
	album : STRING is
		do
			Result := string_tag_query("ALBUM", 0)
		end

	
	annee : INTEGER is
		local
			string_year : STRING
		do
			string_year := string_tag_query("YEAR", 0)
			if (string_year.is_integer) then
				Result := string_year.to_integer
			else
				Result := 0
			end
		end

	
	genre : INTEGER is
		local
			tmp_string : STRING
		do
			tmp_string := string_tag_query("GENRE", 0)
			if (tmp_string.is_integer) then
				Result := tmp_string.to_integer
			else
				Result := 0
			end
		end

	
	comment : STRING is
		do
			Result := string_tag_query("COMMENT", 0)
		end
	
	
	
end
	
