-- 
-- Written by: Vincent LARK vincent.lark@gmail.com
--             Julien COLIN
--   
--   
-- This program is free software; you can redistribute it and/or
-- modify it under the terms of the GNU Library General Public License
-- as published by the Free Software Foundation; either version 2 of
-- the License, or (at your option) any later version.
-- 
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU Library General Public License for more details.
-- 
-- You should have received a copy of the GNU Library General Public
-- License along with this program; if not, write to the Free Software
-- Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
--

class TEXT_GAME

insert ARGUMENTS
	
creation
	make
	
	
feature

	audio_collection : AUDIO_COLLECTION
	-- Blibliotheque audio associe au jeu

	audioutils : AUDIOUTILS
	-- Objet AUDIO_UTILS

	game_system : GAME
	-- Objet Jeu

	

	make is
	-- Mthode main du jeu en mode texte
		local
			i, nb_choices : INTEGER
			path : STRING
			system : SYSTEM
			response : STRING
			basic_directory : BASIC_DIRECTORY

			make_audiocollection_function : PROCEDURE[ANY, TUPLE[STRING]]
			-- Code report pour crer la collection

		do
			create audioutils.make

			path := Void
			nb_choices := -1


			-- On teste les arguments
			from
				i := 1
			until
				i > argument_count.min(2)
			loop

				-- S'il s'agit d'un entier ...
				if argument(i).is_integer then

					-- ... infrieur  20 ...
					if argument(i).to_integer.in_range(2, 20) then
						-- ... on assigne le nb de choix possibles
						nb_choices := argument(i).to_integer
					else
						io.put_string("Le nombre de choix (")
						io.put_string(argument(i))
						io.put_string(") n'est pas valide.")
						io.put_new_line
					end
					
				else -- ... Sinon ...

					-- ... s'il s'agit de --help (ou simmilaires)
					if argument(i).same_as("-h") or
						argument(i).same_as("--h") or
						argument(i).same_as("-help") or
						argument(i).same_as("--help")
					 then

						Current.usage
						die_with_code(exit_success_code)
						
					else -- ... Sinon, ...

						-- ... Il faut un argument i + 1 ...
						if argument_count < i + 1 then

							Current.usage
							die_with_code(exit_failure_code)

						else -- ... pour assigner un emplacement
							  -- de fichiers musicaux...

							
							if argument(i).same_as("-p") then
								
								--  il s'agit d'une playlist  
								path := argument(i + 1)
								make_audiocollection_function := agent create_with_playlist(?)
								i := i + 1
								
							elseif argument(i).same_as("-d") then

								--  il s'agit d'un rpertoire
								path := argument(i + 1)
								-- on active la rcursivit
								make_audiocollection_function := agent create_with_path(?, True)
								i := i + 1

							else -- ... si ni '-d' , ni '-p', on quittte
								
								Current.usage
								die_with_code(exit_failure_code)

							end

						end 
						
					end
						
				end
					
				i := i + 1
				
			end -- Fin du test des arguments.

			-- Si aucune source de fichier n'a t donn,
			-- on donne le rpertoire courant
			if path = Void then
				path := basic_directory.current_working_directory
				make_audiocollection_function := agent create_with_path(?, True)
		
				--Current.usage
				--die_with_code(exit_failure_code)
			end

			-- Si nb_choices n'a t modifi,
			-- ou si nb_choices < 2,
			if nb_choices < 2 then
				-- On donne 4 choix
				nb_choices := 4
			end

			-- Juste pour dire  partire de quoi
			-- se base la collection.
			io.put_new_line
			io.put_string("Construction de la collection a partir de ")
			io.put_string(path)
			io.put_string(",")
			io.put_new_line

			-- Execution du code report,
			-- pour crer la collection
			make_audiocollection_function.call([path])
			
			io.put_string(" soit ")
			io.put_integer(game_system.count)
			io.put_string(" fichiers.")
			io.put_new_line


			-- Si le nombre de fichiers n'est pas
			-- suffisant, ou si le nombre de choix
			-- excede le nombre de fichiers,
			-- on quitte avec un message
			if game_system.count < 10
				or nb_choices > game_system.count
			 then
				io.put_string("Il n'y a pas assez de fichiers musicaux -> EXIT")
				io.put_new_line
				die_with_code(exit_failure_code)
			end



			-- On prvient que le score initial est de 2
			io.put_string("Vous avez un score initial de 2,")
			io.put_new_line
			io.put_string(" une bonne reponse vous donne 1 point")
			io.put_new_line
			io.put_string(" une mauvaise vous en retire 1 !")
			io.put_new_line
			
			
			-- Tant que le jeu n'est pas fini ...
			from
			until game_system.is_game_finished
			loop

				-- On initialise un nouveau choix
				-- avec nb_choices choix
				game_system.init_game_choices(nb_choices)
				
				-- On teste si l'initialisation s'est bien pass
				if not game_system.is_game_finished then
					
					io.put_new_line
					io.put_string("#############################")
					io.put_new_line

					from i := game_system.random_titles_array.lower
					until i = game_system.random_titles_array.upper
					loop
						io.put_string("Choix ")
						io.put_integer(i+1)
						io.put_string(" : ")
						io.put_string(game_system.random_titles_array.item(i))
						io.put_new_line
						
						i := i + 1
					end
					
					io.put_new_line				
					io.put_string("Lancement du titre aleatoire ... ")
					io.put_new_line
					
					create response.make(0)
			
					game_system.play_sample

					if game_system.sample_playing then
						
						ask("Entrez la reponse, ou 'q' pour quitter : ", response)
						
						game_system.stop_sample

						if response.same_as("q") then
							
							post_correct_game_stuff
							die_with_code(exit_success_code)

						end
						
						if (game_system.is_the_answer(response.to_integer - 1)) then
							io.put_string("  BRAVO !!    essais restants = " + game_system.score.to_string)
							io.put_new_line
						else
							io.put_string("  FAUX  !!    essais restants = " + game_system.score.to_string)
							io.put_new_line
						end
					
						
					else -- if not sample_playing ...
						
						io.put_string("Erreur de lecture !")
						io.put_new_line
					end
					

					if game_system.sample_playing then
						game_system.stop_sample
					end
					io.put_new_line
					io.put_new_line

				end
			end


			
			if game_system.last_good_index = -1 then
				
				io.put_string("Il n'a pas ete possible de trouver ")
				if game_system.nb_try /= -1 then
					io.put_string("(en ")
					io.put_integer(game_system.nb_try)
					io.put_string(" tentatives) ")
				end
				io.put_string("un fichier correctement taggue, peut-etre que votre audiotheque ne contient pas assez de fichiers taggues !")
				
			else

				io.put_string("Vous n'avez plus d'esais, recommencez !")
				io.put_new_line
				io.put_new_line
				
				post_correct_game_stuff
				
			end
			io.put_new_line
		end
	



	usage is
	-- Affiche une petite aide
		do
			io.put_new_line
			io.put_string("SmartEiffel MusicQuizzzzz            %N")
			io.put_string("Usage :                              %N")
			io.put_string("./game [nb_choices] [-d playlist | -p directory]%N")
			io.put_new_line
			io.put_string("%T -d directory   Repertoire de fichiers musicaux MP3 ou OGG.       %N")
			io.put_string("%T                tous les sous repertoires seront ajoutes.         %N")
			io.put_new_line
			io.put_string("%T -p playlist    Fichier texte contenant un chemin (vers un fichier%N")
			io.put_string("%T                MP3 ou OGG) par ligne, comme les fichiers .m3u    %N")
			io.put_new_line
			io.put_string("%T nb_choices     Nombre de choix (4 par defaut)                    %N")
			io.put_new_line
		end


	
	
	
feature {NONE}


	with_path : INTEGER is 1
	with_list : INTEGER is 2
	create_with : INTEGER
	
	create_with_playlist(path : STRING) : INTEGER is
	-- Fonction de cration du jeu,
	--  partir d'une playlist
		do
			create game_system.make_with_playlist(path)
		end



	create_with_path(path : STRING ; recursive : BOOLEAN) : INTEGER is
	-- Fonction de cration du jeu,
	--  partir d'un rpertoire
		do
			create game_system.make_with_path(path, recursive)
		end

	
	
	valid_answer(response : STRING) : BOOLEAN is
	-- Teste si cette rponse peut tre accepte ou non
		require
			response /= Void
		do
			if not response.is_empty then
				if response.is_integer then
					if response.to_integer.in_range(game_system.random_titles_array.lower,
															  game_system.random_titles_array.upper)
					 then
						Result := True
					end
				else
					if response.same_as("q") then
						Result := True
					end
				end
			end
		end
	

	
	ask(question : STRING ; response : STRING) is
	-- Lire la rponse
		require
			question /= Void
			not question.is_empty
			response /= Void
		local
			i : INTEGER
		do
			from i := 0
			until i = 2
			loop
				io.put_string(question)
				io.flush
				response.wipe_out
				io.read_line_in(response)
				io.put_new_line

				if valid_answer(response) then
					i := 2
				end
			end
			
			if not valid_answer(response) then
				io.put_string("Vous n'avez pas entre de reponse valide !")
				io.put_new_line
				die_with_code(exit_failure_code)
			end
			
			--if response.same_as("q") then
			--	if game_system.sample_playing then
			--		game_system.stop_sample
			--	end
			--	die_with_code(exit_success_code)
			--end
		end

	

	post_correct_game_stuff is
	-- Code commun aux sorties (correctes) du jeu
		local
			response : STRING		
		do
			create response.make_empty
			
			test_and_update_highscores
			
			io.put_string("Voulez vous voir votre score ? ('non' a defaut) ")
			io.flush
			response.wipe_out
			io.read_line_in(response)

			if not response.is_empty then
				view_score
			end

			io.put_new_line
			
			io.put_string("Voulez vous voir les meilleurs scores ? ('non' a defaut) ")
			io.flush
			response.wipe_out
			io.read_line_in(response)

			if not response.is_empty then
				view_highscores
			end

		end
	

	
	view_score is
	-- Voir le score
		do
			io.put_string("Vos resultats :")
			io.put_new_line
			io.put_string(game_system.nb_good_responses.to_string + "/"  +
							  game_system.nb_responses.to_string      + " (" +
							  (game_system.nb_good_responses	/
								game_system.nb_responses * 100).to_string_format(1) + " %%)")
			io.put_new_line
		end	


	
	test_and_update_highscores is
	-- Tester s'il on atteint un reccord, et mettre,
	-- le cas chant, les highscore  jour
		local
			time : TIME
			response : STRING
		do
			if game_system.is_an_highscore then
				io.put_string("C'est un reccord ! Entrez votre nom : ")
				create response.make_empty
				io.read_line_in(response)
				
				if response.is_empty then
					time.update
					response :=	time.day.to_string + "-" + time.month.to_string

				elseif response.count > 8 then
					response.keep_head(8)
				end
				
				game_system.write_highscore(response)
			end
		end
			

	
	view_highscores is
	-- Voir les highscores
		local
			i : INTEGER
			high_scores : ARRAY[HIGHSCORE]
		do
			high_scores := game_system.sorted_high_scores
			
			if not high_scores.is_empty then

				io.put_string("-----------< Meilleurs scores >----------")
				io.put_new_line
				io.put_string("NOM (ou date)   Bonnes reponses     Total")
				io.put_new_line

				
				from i := high_scores.lower
				until i > high_scores.upper
				loop
					
					io.put_string(high_scores.item(i).name + "%T%T")
					io.put_string(high_scores.item(i).nb_good_responses.to_string + "%T%T%T")
					io.put_string(high_scores.item(i).nb_responses.to_string)

					io.put_new_line
					
					i := i + 1
				end

				io.put_string("-----------< Meilleurs scores >----------")
				io.put_new_line
			end
		end	


	
end

